<?php

namespace Insign\PostCore\Block\Adminhtml\Order\View;

use Exception;
use Insign\PostCore\Helper\TokenHelper;
use Magento\Backend\Block\Template;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Model\OrderRepository;

class StatusInfo extends Template
{
    /**
     * @var OrderRepository
     */
    protected $orderRepository;
    /**
     * @var TokenHelper
     */
    private $tokenHelper;

    /**
     * StatusInfo constructor.
     * @param OrderRepository $orderRepository
     * @param Context $context
     * @param TokenHelper $tokenHelper
     * @param array $data
     */
    public function __construct(
        OrderRepository $orderRepository,
        Context $context,
        TokenHelper $tokenHelper,
        array $data = []
    ) {
        $this->orderRepository = $orderRepository;
        $this->tokenHelper = $tokenHelper;
        parent::__construct($context, $data);
    }

    /**
     * @return mixed
     * @throws InputException
     * @throws NoSuchEntityException
     */
    public function getAddressValidationStatus()
    {
        $order_id = $this->getRequest()->getParam('order_id');
        try {
            $order = $this->orderRepository->get($order_id);
            if (!empty($order->getShippingAddress())) {
                return $order->getShippingAddress()->getAddressValidationStatus();
            }
        } catch (Exception $e) {
            throw  $e;
        }
    }

    /**
     * @return string
     * @throws InputException
     * @throws NoSuchEntityException
     */
    public function getMessage(): string
    {
        if (!$this->tokenHelper->isValidationEnable() || !($status = $this->getAddressValidationStatus())) {
            return false;
        };

        switch ($status) {
                    case "CERTIFIED":
                    case "DOMICILE_CERTIFIED":
                        return "The delivery address was successfully validated";
                    case "USABLE":
                    case "FIXED":
                        return "The delivery address is valid but could not be verified.";
                    case "UNUSABLE":
                    case "COMPROMISED":
                        return "The validation of the delivery address has shown that it is invalid.";
                    default:
                        return 'An error occurred during the validation of the delivery address.';
                }
    }

    /**
     * @return string
     * @throws InputException
     * @throws NoSuchEntityException
     */
    public function getMessageType(): string
    {
        if (!$this->tokenHelper->isValidationEnable() || !($status = $this->getAddressValidationStatus())) {
            return false;
        };

        switch ($status) {
            case "CERTIFIED":
            case "DOMICILE_CERTIFIED":
                return "message2 post-success";
            case "USABLE":
            case "FIXED":
                return "message2 post-info";
            case "UNUSABLE":
            case "COMPROMISED":
                return "message2 post-warning";
            default:
                return "message2 post-error";
        }
    }

}
