<?php

namespace Insign\PostCore\Controller\Adminhtml\Barcode;

use Exception;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\Barcode;
use Insign\PostCore\Model\BarcodeFactory;
use Insign\PostCore\Model\ResourceModel\Barcode\BarcodeCollectionFactory;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Checkout\Model\Session;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\View\Result\PageFactory;

class Edit extends Action
{

    /**
     * @var PageFactory
     */
    protected $resultPageFactory;
    /**
     * @var Session
     */
    protected $session;

    /**
     * @var BarcodeCollectionFactory
     */
    protected $collectionFactory;
    /**
     * @var Log
     */
    protected $logger;
    /**
     * @var BarcodeFactory
     */
    private $barcodeFactory;

    /**
     * Edit constructor.
     * @param Context $context
     * @param PageFactory $resultPageFactory
     * @param Session $session
     * @param BarcodeCollectionFactory $collectionFactory
     * @param Log $logger
     * @param BarcodeFactory $barcodeFactory
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        Session $session,
        BarcodeCollectionFactory $collectionFactory,
        Log $logger,
        BarcodeFactory $barcodeFactory
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->session = $session;
        $this->collectionFactory = $collectionFactory;
        $this->logger = $logger;
        $this->barcodeFactory = $barcodeFactory;
    }

    /**
     * @inheritDoc
     * @throws Exception
     */
    public function execute()
    {
        $model = $this->barcodeFactory->create();
        $shipmentId = $this->getRequest()->getParam(Barcode::SHIPPING_ID);
        $this->session->setVarValue($shipmentId);
        $resultPage = $this->resultFactory->create(ResultFactory::TYPE_PAGE);
        $resultPage->getConfig()->getTitle()->prepend(__('Barcode'));
        return $resultPage;

    }

    protected function _isAllowed(): bool
    {
        return $this->_authorization->isAllowed('Insign_PostCore::insign_swiss_post_barcode');
    }
}
