<?php
namespace Insign\PostCore\Model;

use Exception;
use Insign\PostCore\Api\BarcodeRepositoryInterface;
use Insign\PostCore\Api\Data\BarcodeInterface;
use Insign\PostCore\Api\Data\BarcodeSearchResultInterface;
use Insign\PostCore\Api\Data\BarcodeSearchResultInterfaceFactory;
use Insign\PostCore\Model\ResourceModel\Barcode\BarcodeCollectionFactory;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Framework\Exception\NoSuchEntityException;

class BarcodeRepository implements BarcodeRepositoryInterface
{
    /** @var BarcodeFactory  */
    private $barcodeFactory;

    /** @var BarcodeCollectionFactory  */
    private $barcodeCollectionFactory;

    /** @var BarcodeSearchResultInterfaceFactory  */
    private $searchResultFactory;

    /** @var CollectionProcessorInterface */
    private $collectionProcessor;

    /** @var \Insign\PostCore\Model\ResourceModel\Barcode  */
    private $resource;

    public function __construct(
        BarcodeFactory $barcodeFactory,
        \Insign\PostCore\Model\ResourceModel\Barcode $resource,
        BarcodeCollectionFactory $barcodeCollectionFactory,
        BarcodeSearchResultInterfaceFactory $searchResultFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->barcodeFactory = $barcodeFactory;
        $this->resource = $resource;
        $this->barcodeCollectionFactory = $barcodeCollectionFactory;
        $this->searchResultFactory = $searchResultFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @param int $id
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getById(int $id)
    {
        $barcode = $this->barcodeFactory->create();
        $this->resource->load($barcode, $id);
        if (! $barcode->getId()) {
            throw new NoSuchEntityException(__('Unable to find barcode with ID "%1"', $id));
        }
        return $barcode;
    }

    /**
     * @param BarcodeInterface $barcode
     * @return BarcodeInterface
     * @throws AlreadyExistsException
     */
    public function save(BarcodeInterface $barcode): BarcodeInterface
    {
        $barcode->getResource()->save($barcode);
        return $barcode;
    }

    /**
     * @param BarcodeInterface $barcode
     * @throws Exception
     */
    public function delete(BarcodeInterface $barcode)
    {
        $barcode->getResource()->delete($barcode);
    }


    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return BarcodeSearchResultInterface
     */
    public function getList(SearchCriteriaInterface $searchCriteria): BarcodeSearchResultInterface
    {
        $collection = $this->barcodeCollectionFactory->create();
        $this->collectionProcessor->process($searchCriteria, $collection);
        $searchResults = $this->searchResultFactory->create();

        $searchResults->setSearchCriteria($searchCriteria);
        $searchResults->setItems($collection->getItems());

        return $searchResults;
    }
}
