<?php

namespace Insign\PostCore\Model\Config\Source\Barcode;

use Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\View\Element\BlockInterface;

/**
 * @api
 * @since 100.0.2
 */
class ServiceCodes extends AbstractFieldArray
{
    private $shipMethodsRenderer;
    private $deliveryMethodsRenderer;
    /**
     * Prepare rendering the new field by adding all the needed columns
     */
    protected function _prepareToRender()
    {
        $this->addColumn('shipping_methods', [
            'label' => __('Shipping Methods'),
            'class' => 'required-entry',
            'renderer' => $this->getShipMethodsRenderer()
        ]);
        $this->addColumn('service_code', [
            'label' => __('Service Code'),
            'class' => 'required-entry',
            'renderer' => $this->getDeliveryMethodsRenderer()
            ]);
        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
    }

    /**
     * Prepare existing row data object
     *
     * @param DataObject $row
     * @throws LocalizedException
     */
    protected function _prepareArrayRow(DataObject $row): void
    {
        $options = [];

        $tax = $row->getShipMethods();
        if ($tax !== null) {
            $options['option_' . $this->getShipMethodsRenderer()->calcOptionHash($tax)] = 'selected="selected"';
        }

        $row->setData('option_extra_attrs', $options);
    }

    private function getShipMethodsRenderer(): BlockInterface
    {
        if (!$this->shipMethodsRenderer) {
            try {
                $this->shipMethodsRenderer = $this->getLayout()->createBlock(
                    ShippingMethodColumn::class,
                    '',
                    ['data' => ['is_render_to_js_template' => true]]
                );
            } catch (LocalizedException $e) {
                throw new Exception("Error: " . $e->getMessage());
                // add log entry
            }
        }
        return $this->shipMethodsRenderer;
    }

    private function getDeliveryMethodsRenderer(): BlockInterface
    {
        if (!$this->deliveryMethodsRenderer) {
            try {
                $this->deliveryMethodsRenderer = $this->getLayout()->createBlock(
                    DeliveryMethodsColumn::class,
                    '',
                    ['data' => ['is_render_to_js_template' => true]]
                );
            } catch (LocalizedException $e) {
                throw new Exception("Error: " . $e->getMessage());
                // add log entry
            }
        }
        return $this->deliveryMethodsRenderer;
    }
}
